/*

*/

import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.SwingUtilities;
import java.awt.event.ActionEvent;
import java.math.BigInteger;
import static java.util.concurrent.TimeUnit.SECONDS;

/**
   The Event Dispatch Thread (EDT) used by the Java GUI
   framework is essentially a one-thread thread pool.

   When long running computations get submitted (invoked)
   on the EDT the GUI becomes unresponsive. Try clicking
   on the button while the EDT is computing several prime
   factorizations.
*/
public class GUIExperiment
{
   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      //Build the GUI (on the EDT).
      SwingUtilities.invokeLater(() -> new GUIExperiment());

      System.out.println("Waiting 10 seconds.");
      try{SECONDS.sleep(10);}catch(InterruptedException e){}

      System.out.println("Submitting five prime number tasks to the EDT.");
      SwingUtilities.invokeLater(()->System.out.println(Primes.factorize(1234756789123127L)));
      SwingUtilities.invokeLater(()->System.out.println(Primes.factorize(9876543217123125L)));
      SwingUtilities.invokeLater(()->System.out.println(Primes.factorize(99456779999123127L)));
      SwingUtilities.invokeLater(()->System.out.println(Primes.factorize(999997999999123127L)));
      SwingUtilities.invokeLater(()->System.out.println(Primes.factorize(new BigInteger("898799891293129983799597"))));
      System.out.println("main() method done.");
   }


   public GUIExperiment()
   {
      final JFrame frame = new JFrame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setSize(250, 250);
      frame.setLocationRelativeTo(null);
      frame.setVisible(true);

      final JButton jb = new JButton("Hi");
      frame.add(jb);
      jb.addActionListener(
         (ActionEvent e)->{
            System.out.println("Ouch.");
            SwingUtilities.invokeLater(
               ()->System.out.println("Ouch, ouch!"));});
   }
}
